/*
 *
 *  Copyright (C) 1998-99, OFFIS
 *
 *  This software and supporting documentation were developed by
 *
 *    Kuratorium OFFIS e.V.
 *    Healthcare Information and Communication Systems
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND OFFIS MAKES NO  WARRANTY
 *  REGARDING  THE  SOFTWARE,  ITS  PERFORMANCE,  ITS  MERCHANTABILITY  OR
 *  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES  OR
 *  ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND
 *  PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Module: imagectn
 *
 *  Author: Lutz Vorwerk
 *
 *  Purpose:
 *    enables access to functions of dbstore.cc
 *
 *  Last Update:      $Author: vorwerk $
 *  Update Date:      $Date: 1998/12/22 15:25:34 $
 *  CVS/RCS Revision: $Revision: 1.1 $
 *  Status:           $State: Exp $
 *
 *  CVS/RCS Log at end of file
 *
 */


#ifndef __dbstore_H
#define __dbstore_H


#include "osconfig.h"    /* make sure OS specific configuration is included first */

BEGIN_EXTERN_C
#include <stdio.h>
#include <errno.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#ifdef HAVE_TIME_H
#include <time.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
END_EXTERN_C

#include "dcompat.h"
#include "imagedb.h"
#include "dbpriv.h"
#include "dcfilefo.h"
#include "dimse.h"


/** Image file deleting 
*/

CONDITION DB_deleteImageFile(char* imgFile);

/** Delete oldest study in database  
 */

extern int
DB_DeleteOldestStudy(DB_Private_Handle *phandle, StudyDescRecord *pStudyDesc);

/**	 Delete oldest images in database  
 */

CONDITION
DB_DeleteOldestImages(DB_Private_Handle *phandle, StudyDescRecord *pStudyDesc, int StudyNum, char *StudyUID, long RequiredSize); 

/**	 Verify if study UID already exists  
 */

extern int
DB_MatchStudyUIDInStudyDesc (StudyDescRecord *pStudyDesc, char *StudyUID, int maxStudiesAllowed);

/**	Check up storage rights in Study Desk record 
 */

CONDITION
DB_CheckupinStudyDesc(DB_Private_Handle *phandle, StudyDescRecord *pStudyDesc, char *StudyUID, long imageSize);


/** If the image is already stored remove it from the database.
 
 */

CONDITION DB_removeDuplicateImage(DB_Private_Handle *phandle, 
	const char *SOPInstanceUID, const char *StudyInstanceUID, 
	StudyDescRecord *pStudyDesc, const char *newImageFileName);


/**	Add data from imageFileName to database
 */

CONDITION 
DB_storeRequest (
	DB_Handle 	*handle, 
	const char 	*SOPClassUID, 
	const char 	* /*SOPInstanceUID*/, 
	const char 	*imageFileName, 
	DB_Status 	*status);


/** Prune invalid DB records.
*/

CONDITION DB_pruneInvalidRecords(DB_Handle *dbHandle);
#endif

/*
 *  $Log: dbstore.h,v $
 *  Revision 1.1  1998/12/22 15:25:34  vorwerk
 *  added for access of functions from imagectn/libsrc/dbstore.cc
 *
 *
 */
