/*
 *
 *  Copyright (C) 1996-99, OFFIS
 *
 *  This software and supporting documentation were developed by
 *
 *    Kuratorium OFFIS e.V.
 *    Healthcare Information and Communication Systems
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND OFFIS MAKES NO  WARRANTY
 *  REGARDING  THE  SOFTWARE,  ITS  PERFORMANCE,  ITS  MERCHANTABILITY  OR
 *  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES  OR
 *  ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND
 *  PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Module:  wlistctn
 *
 *  Author:  Andrew Hewett
 *
 *  Purpose: Common Constants, Types, Globals and Functions
 *
 *  Last Update:      $Author: meichel $
 *  Update Date:      $Date: 1999/04/26 12:59:13 $
 *  Source File:      $Source: /share/dicom/cvs-depot/dcmtk/wlistctn/apps/wlistctn.h,v $
 *  CVS/RCS Revision: $Revision: 1.5 $
 *  Status:           $State: Exp $
 *
 *  CVS/RCS Log at end of file
 *
 */

#ifndef WLISTCTN_H
#define WLISTCTN_H


#include "osconfig.h"    /* make sure OS specific configuration is included first */

#include "dicom.h"
#include "cond.h"
#include "dctypes.h"
#include "dcxfer.h"

/*
 * Application Conditions
 */

#define APP_NORMAL		/* normal operation */\
	FORM_COND(FAC_APP, SEV_SUCC, 0)
#define APP_BADIMAGE		/* bad image data */\
	FORM_COND(FAC_APP, SEV_ERROR, 1)
#define APP_NOCONTEXT		/* no presentation context for SOP Class */\
	FORM_COND(FAC_APP, SEV_ERROR, 2)
#define APP_PROTOCOLERROR	/* protocol error */\
	FORM_COND(FAC_APP, SEV_ERROR, 3)
#define APP_MESSAGEFAILED	/* network message failed */\
	FORM_COND(FAC_APP, SEV_ERROR, 4)
#define APP_STOREFAILED		/* store operation failed */\
	FORM_COND(FAC_APP, SEV_ERROR, 5)
#define APP_INVALIDPEER		/* invalid peer */\
	FORM_COND(FAC_APP, SEV_ERROR, 6)

/*
 * Common Globals (defined in wlistctn.c)
 */

extern const char* progname;
extern int opt_verbose;
extern int opt_debug;

extern const char* baseWorklistDBPath;

extern unsigned long     opt_sleepAfterFind;
extern unsigned long     opt_sleepDuringFind;
extern E_TransferSyntax  opt_networkTransferSyntax;
extern E_GrpLenEncoding  opt_groupLength;
extern E_EncodingType    opt_sequenceType;
extern E_PaddingEncoding opt_paddingType;

extern int opt_failInvalidQuery;

/*
 * Common Function Definitions
 */

extern void errmsg(const char* msg, ...);


#endif

/*
** CVS Log
** $Log: wlistctn.h,v $
** Revision 1.5  1999/04/26 12:59:13  meichel
** Ported Worklist CTN to Win32 environment.
**
** Revision 1.4  1998/07/24 13:30:21  meichel
** Minor changes to avoid compiler warnings (gcc 2.8.1 with additional
**   options), e.g. add copy constructors.
**
** Revision 1.3  1997/07/21 09:08:53  andreas
** - Replace all boolean types (BOOLEAN, CTNBOOLEAN, DICOM_BOOL, BOOL)
**   with one unique boolean type OFBool.
**
** Revision 1.2  1996/08/29 16:16:10  hewett
** Corrected copyright header.
**
** Revision 1.1.1.1  1996/04/26 14:16:01  hewett
** Initial Import.
**
*/
